"""
# - iterate over datasets
#   - group in society sets and others
#   - aggregate data grouped by society set
# - write out package per society set
"""


def run(args):
    raise NotImplementedError()
